#!/bin/sh

#UST scalability test

BINARY=tracepoint_benchmark
REPORT=/tmp/testreport
TMPLOG=/tmp/testlog
WRAPPER=""
STAP=stap
STAPTMP=/tmp/stapconsole
STAPPROBE=testutrace.stp

rm $REPORT 2>/dev/null

ust_flight_recorder() {
	# flight recorder, don't record trace to disk.
	# default buffer size is 4k
	echo -n "* UST Flight recorder : " | tee >> $REPORT
	export UST_AUTOCOLLECT=0
	export UST_OVERWRITE=1
	export UST_SUBBUF_NUM=16
	WRAPPER=usttrace
}

ust_disk() {
	# Collect traces to disk
	# default buffer size is 4k
	echo -n "* UST Write to disk : " | tee >> $REPORT
	export UST_AUTOCOLLECT=1
	export UST_OVERWRITE=0
	export UST_SUBBUF_NUM=16
	WRAPPER=usttrace
}

stap_flight_recorder() {
	echo -n "* SystemTap Flight recorder : " | tee >> $REPORT
	WRAPPER=""
	$STAP $STAPPROBE -F -m $BINARY
}

stap_disk() {
	echo -n "* SystemTap Write to disk : " | tee >> $REPORT
	WRAPPER=""
	$STAP $STAPPROBE -o $STAPTMP -m $BINARY &
	sleep 5
}

echo "Userspace tracing scalability test report" |tee >> $REPORT
case "$1" in
	ust_flight_recorder)
		TEST=ust_flight_recorder
		;;
	ust_disk)
		TEST=ust_disk
		;;
	stap_flight_recorder)
		TEST=stap_flight_recorder
		;;
	stap_disk)
		TEST=stap_disk
		;;
	*)
		echo "Usage : $0 {ust_flight_recorder|ust_disk|stap_flight_recorder|stap_disk}"
		exit 1
		;;
esac

for nr_threads in 1 2 4 8; do
	echo "" | tee >> $REPORT
	echo Number of threads: $nr_threads | tee >> $REPORT
	echo -n "* Baseline : " | tee >> $REPORT

	# just some cleanup
	killall stapio 2>/dev/null
	rmmod $BINARY 2>/dev/null

	sync
	/usr/bin/time -f "%E" -o $TMPLOG ./$BINARY ${nr_threads}
	cat $TMPLOG >> $REPORT

	$TEST

	sync
	/usr/bin/time -f "%E" -o $TMPLOG $WRAPPER ./$BINARY ${nr_threads}
	cat $TMPLOG >> $REPORT
done

cat $REPORT

